@@ -46,7 +46,7 @@ Expression* domain_shallow_copy(EnvI& env, Expression* orig, Type type) {
                             domain_shallow_copy(env, ti->domain(), nt));
   }
   ArrayLit* tup = ArrayLit::constructTuple(Expression::loc(orig), clone);
-  tup->type(type);
+  tup->type(type.elemType(env));
   return tup;
 }
 
@@ -1265,7 +1265,9 @@ void TypeInst::setStructDomain(EnvI& env, const Type& struct_type, bool setTypeA
     }
   }
   if (setTIRanges && ranges().size() != type().dim()) {
-    assert(ranges().empty());
+    assert(ranges().empty() || ranges().size() == 1 &&
+                                   Expression::isa<TIId>(ranges()[0]->domain()) &&
+                                   !Expression::cast<TIId>(ranges()[0]->domain())->isEnum());
     std::vector<TypeInst*> newRanges(type().dim());
     for (unsigned int k = 0; k < type().dim(); k++) {
       newRanges[k] = new TypeInst(Location().introduce(), Type::parint());
@@ -1312,6 +1314,12 @@ bool TypeInst::resolveAlias(EnvI& env) {
   }
   if (type().otExplicit() && ntype.ot() != type().ot()) {
     if (type().ot() == Type::OT_OPTIONAL) {
+      if (ntype.bt() == Type::BT_TUPLE) {
+        throw TypeError(env, Expression::loc(this), "opt tuples are not allowed");
+      }
+      if (ntype.bt() == Type::BT_RECORD) {
+        throw TypeError(env, Expression::loc(this), "opt records are not allowed");
+      }
       ntype.mkOpt(env);
     } else {
       ntype.mkPresent(env);
@@ -1408,12 +1416,12 @@ bool TypeInst::concatDomain(EnvI& env) {
     assert(lhs->type().istuple());
     assert(rhs->type().istuple());
     GCLock lock;
-    // Concat domains
-    auto* nbo = new BinOp(Expression::loc(bop), lhs->domain(), bop->op(), rhs->domain());
-    nbo->type(bop->type());
-    dom = ArrayLit::constructTuple(Expression::loc(bop).introduce(), eval_array_lit(env, nbo));
     // Concat types
     ty = env.concatTuple(lhs->type(), rhs->type());
+    // Concat domains
+    auto* nbo = new BinOp(Expression::loc(bop), lhs->domain(), bop->op(), rhs->domain());
+    nbo->type(ty);
+    dom = ArrayLit::constructTuple(Expression::loc(bop).introduce(), eval_array_lit(env, nbo));
     dom->type(ty);
   }
   // Update TI
@@ -1675,6 +1683,7 @@ Type type_from_tmap(EnvI& env, TypeInst* ti, const ASTStringMap<std::pair<Type,
       if (ret.any()) {
         ret.ot(it->second.first.ot());
         ret.ti(it->second.first.ti());
+        ret.cv(it->second.first.cv());
         ret.any(false);
       }
     }
@@ -2184,9 +2193,12 @@ Constants::Constants() {
   ids.arrayXd = addString("arrayXd");
 
   ids.sum = addString("sum");
+  ids.lex_less = addString("lex_less");
+  ids.lex_lesseq = addString("lex_lesseq");
   ids.lin_exp = addString("lin_exp");
   ids.count = addString("count");
   ids.element = addString("element");
+  ids.table = addString("table");
   ids.anon_enum = addString("anon_enum");
   ids.anon_enum_set = addString("anon_enum_set");
   ids.enumFromConstructors = addString("enumFromConstructors");
