@@ -297,8 +297,9 @@ void unify(EnvI& env, std::vector<VarDecl*>& deletedVarDecls, Id* id0, Id* id1)
       assert(env.outputFlatVarOccurrences.find(id1->decl()) != -1);
       VarDecl* id1_output =
           (*env.output)[env.outputFlatVarOccurrences.find(id1->decl())]->cast<VarDeclI>()->e();
-      if (id0_output->e() == nullptr) {
-        id0_output->e(id1_output->id());
+      auto* decl = Expression::cast<VarDecl>(follow_id_to_decl(id0_output));
+      if (decl->e() == nullptr) {
+        decl->e(id1_output->id());
       }
     }
 
@@ -360,6 +361,8 @@ void remove_deleted_items(EnvI& envi, std::vector<VarDecl*>& deletedVarDecls) {
     if (envi.varOccurrences.occurrences(cur) == 0) {
       auto cur_idx = envi.varOccurrences.idx.find(cur->id());
       if (cur_idx.first && !m[*cur_idx.second]->removed()) {
+        auto* vdi = m[*cur_idx.second]->cast<VarDeclI>();
+        cur = vdi->e();
         if (is_output(cur)) {
           // We have to change the output model if we remove this variable
           Expression* val = nullptr;
@@ -399,16 +402,14 @@ void remove_deleted_items(EnvI& envi, std::vector<VarDecl*>& deletedVarDecls) {
             VarDecl* vd_out =
                 (*envi.output)[envi.outputFlatVarOccurrences.find(cur)]->cast<VarDeclI>()->e();
             vd_out->e(val);
-            CollectDecls cd(envi, envi.varOccurrences, deletedVarDecls,
-                            m[*cur_idx.second]->cast<VarDeclI>());
+            CollectDecls cd(envi, envi.varOccurrences, deletedVarDecls, vdi);
             top_down(cd, cur->e());
-            (*envi.flat())[*cur_idx.second]->remove();
+            vdi->remove();
           }
         } else {
-          CollectDecls cd(envi, envi.varOccurrences, deletedVarDecls,
-                          m[*cur_idx.second]->cast<VarDeclI>());
+          CollectDecls cd(envi, envi.varOccurrences, deletedVarDecls, vdi);
           top_down(cd, cur->e());
-          (*envi.flat())[*cur_idx.second]->remove();
+          vdi->remove();
         }
       }
     }
